// ASEC.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <windows.h>
#include <stdlib.h>
#include "bfm.h"



#define NUM_VERTEX			"*MESH_NUMVERTEX"
#define NUM_FACE			"*MESH_NUMFACES"
#define NUM_TVERTEX			"*MESH_NUMTVERTEX"
#define NUM_TFACE			"*MESH_NUMTVFACES"
#define VERTEX_LIST			"*MESH_VERTEX_LIST"
#define VERTEX				"*MESH_VERTEX"
#define FACE_LIST			"*MESH_FACE_LIST"
#define FACE				"*MESH_FACE"
#define NORMALS				"*MESH_NORMALS"
#define FACE_NORMAL			"*MESH_FACENORMAL"
#define NVERTEX				"*MESH_VERTEXNORMAL"
#define TVERTEX				"*MESH_TVERT"
#define TFACE				"*MESH_TFACE"
#define TEXTURE				"*BITMAP"
#define UTILE				"*UVW_U_TILING"
#define VTILE				"*UVW_V_TILING"
#define UOFFSET				"*UVW_U_OFFSET"
#define VOFFSET				"*UVW_V_OFFSET"
#define GEOMOBJECT			"*GEOMOBJECT"
#define NODENAME			"*NODE_NAME"
#define WIREFRAMECOLOR		"*WIREFRAME_COLOR"
#define GROUP				"*GROUP"
#define MAT_COUNT			"*MATERIAL_COUNT"
#define MAT_MATERIAL		"*MATERIAL"
#define MAT_REFERENCE		"*MATERIAL_REF"
#define MAT_SUBMATERIAL		"*SUBMATERIAL"
#define MAT_NUMSUBMTL		"*NUMSUBMTLS"
#define MAT_AMBIENT			"*MATERIAL_AMBIENT"
#define MAT_DIFFUSE			"*MATERIAL_DIFFUSE"
#define MAT_SPECULAR		"*MATERIAL_SPECULAR"
#define MESH_MTLID			"*MESH_MTLID"



#define SIG_MATERIAL_COUNT			(BFMINT8)1
#define SIG_MATERIAL				(BFMINT8)2
#define SIG_MATERIAL_AMBIENT		(BFMINT8)3
#define SIG_MATERIAL_DIFFUSE		(BFMINT8)4
#define SIG_MATERIAL_SPECULAR		(BFMINT8)5
#define SIG_TEXTURE_NAME			(BFMINT8)6
#define SIG_NODENAME				(BFMINT8)7
#define SIG_WIREFRAMECOLOR			(BFMINT8)8
#define SIG_TEXTURE_UTILE			(BFMINT8)9
#define SIG_TEXTURE_VTILE			(BFMINT8)10
#define SIG_TEXTURE_UOFFSET			(BFMINT8)11
#define SIG_TEXTURE_VOFFSET			(BFMINT8)12
#define SIG_GEOMOBJECT				(BFMINT8)13
#define SIG_NUM_VERTEX				(BFMINT8)14
#define SIG_NUM_FACE				(BFMINT8)15
#define SIG_NUM_TVERTEX				(BFMINT8)16
#define SIG_NUM_TFACE				(BFMINT8)17
#define SIG_VERTEX					(BFMINT8)18
#define SIG_FACE					(BFMINT8)19
#define SIG_TVERTEX					(BFMINT8)20
#define SIG_TFACE					(BFMINT8)21
#define SIG_MATERIAL_REF			(BFMINT8)22
#define SIG_SUBMATERIAL				(BFMINT8)23
#define SIG_NUMSUBMTL				(BFMINT8)24
#define SIG_GROUP					(BFMINT8)25
#define SIG_ENDGROUP				(BFMINT8)26
#define SIG_TEXTURE_RES				(BFMINT8)27
#define SIG_MTLID					(BFMINT8)28



typedef struct
{
	BFMINT8	object_count;
	BFMINT8	mat_count;
	BFMINT16 face_count;
} FILE_HEADER;

FILE_HEADER	fh;


static BFManager bfm_bin;
static unsigned char texture_id = 0;		// texture counter
static BOOL	new_node_name = FALSE;
static LONG reserve_ptr, reserve_ptr2;
static LONG reserve_mat_ref;
static int group_range_count = 0;
static int group_count = 0;
static int group_array[100];





long getfilelength(char *f)
{
	HANDLE hFile;
	long file_size;

	hFile = CreateFile(f,
			GENERIC_READ,
			FILE_SHARE_READ,
			NULL,
			OPEN_EXISTING,
			FILE_ATTRIBUTE_NORMAL,
			NULL);
	
	if (hFile == INVALID_HANDLE_VALUE)
		return -1;

	file_size = GetFileSize(hFile, NULL);

	CloseHandle(hFile);

	return file_size;
}


void import_texture(char* fn)
{
	FILE *fp;
	char *buf;
	long fs;

	fp = fopen(fn, "rb");
	
	printf("importing resource... %s  ", fn);
	
	if (fp == NULL)
	{
		printf("\tERROR: cannot open texture source file.  importing failed...\n");
	}
	
	fs = getfilelength(fn);
	
	if (fs <= 0)
	{
		printf("\tERROR: file (%s) has wrong size.  importing failed...", fn);
		fs = 0;
	}
	
	buf = (char*)malloc(fs);
	fread((void*)buf, 1, fs, fp);
	
	bfm_bin.WriteInt32((BFMINT32*)&fs, 1);
	bfm_bin.WriteInt8((BFMINT8*)buf, fs);
	
	free(buf);

	if (fp != NULL)
		fclose(fp);

	printf("\tOK.\n");
}


void ExtractFileNameOnly(char *src, char *out)
{
	int i, len;

	if ('"' == *src)
		src++;

	len = strlen(src);

	for (i=len; i>0; i--)
		if (src[i] == '\\')
		{
			strcpy(out, src+i+1);
			return;
		}

	strcpy(out, src);
}

void GetTextureName(FILE *s)
{
	char buf[256];
	char obuf[32];
	int len;


	ZeroMemory((void*)buf, 256);
	ZeroMemory((void*)obuf, 32);

	fscanf(s, "%s", buf);

	ExtractFileNameOnly(buf, obuf);

	len = strlen(obuf);
	if (len > 31)
	{
		printf("Fatal error!   Texture file name is too long!  (MAX:31)\n");
		exit(1);
	}

	// to remove "
	if (obuf[len-1] == '"')
		obuf[len-1] = NULL;

	bfm_bin.WriteSig(SIG_TEXTURE_NAME);
	bfm_bin.WriteInt8((PBFMINT8)obuf, 32);

	// write the texture_id
	bfm_bin.WriteInt8((PBFMINT8)&texture_id, 1);

	import_texture(obuf);

	// if texture number exceeds the max, the result will not be defined!   :)
	texture_id++;
}

void aseParse(FILE *s)
{
	char data[255];



	ZeroMemory((void*)group_array, sizeof(int)*100);


	// get file size
	fseek(s, 0, SEEK_END);
	LONG fs = ftell(s);

	printf("read file(%d bytes)...\n\n", fs);

	rewind(s);


	while(!feof(s))
	{
		fscanf(s, "%s", data);

		if (!strcmp(data, GEOMOBJECT))
		{
			fh.object_count++;
			new_node_name = TRUE;
		}

		else if (!strcmp(data, NODENAME))
		{
			if (new_node_name)
			{
				short v = -1;	// default: not use materials

				char buf[MAX_BF_NAME];
				
				fscanf(s, " \"%s", buf);
				
				// to remove "
				buf[strlen(buf)-1] = NULL;
				
				bfm_bin.WriteSig(SIG_NODENAME);
				bfm_bin.WriteInt8((PBFMINT8)buf, MAX_BF_NAME);
				printf("object name: %s (ID: %d)\n", buf, fh.object_count);

				new_node_name = FALSE;

				// reserve for MATERIAL_REF (2)
				bfm_bin.WriteSig(SIG_MATERIAL_REF);
				bfm_bin.WriteCurrent((PBFMINT32)&reserve_mat_ref);
				bfm_bin.WriteInt16((PBFMINT16)&v, 1);
			}
		}
		else if (!strcmp(data, NUM_VERTEX))
		{
			short v;

			fscanf(s, "%d", &v);

			bfm_bin.WriteSig(SIG_NUM_VERTEX);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
			printf("* total vertices: %d\n", v);
		}
		else if (!strcmp(data, NUM_FACE))
		{
			short v;

			fscanf(s, "%d", &v);

			bfm_bin.WriteSig(SIG_NUM_FACE);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
			printf("* total faces: %d\n", v);
		}
		else if (!strcmp(data, NUM_TVERTEX))
		{
			short v;
			fscanf(s, "%d", &v);

			bfm_bin.WriteSig(SIG_NUM_TVERTEX);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
		}
		// read # of texture faces.
		else if (!strcmp(data, NUM_TFACE))
		{
			short v;
			fscanf(s, "%d", &v);

			bfm_bin.WriteSig(SIG_NUM_TFACE);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
		}
		// read wire-frame color
		else if (!strcmp(data, WIREFRAMECOLOR))
		{
			float f[3];
			
			fscanf(s, "%f %f %f", &f[0], &f[1], &f[2]);

			bfm_bin.WriteSig(SIG_WIREFRAMECOLOR);
			bfm_bin.WriteFloat32((PBFMFLOAT32)f, 3);
			printf("* wire frame color: %f, %f, %f\n", f[0], f[1], f[2]);
		}
		// read vertex
		else if (!strcmp(data, VERTEX))
		{
			short v;
			float f[3];
			
			fscanf(s, "%d", &v);
			fscanf(s, "%f %f %f", &f[0], &f[1], &f[2]);

			bfm_bin.WriteSig(SIG_VERTEX);
			bfm_bin.WriteFloat32((PBFMFLOAT32)f, 3);
			printf("* vtx %d: %f, %f, %f\n", v, f[0], f[1], f[2]);
		}
		// read face
		else if (!strcmp(data, FACE))
		{
			short v;
			short vv[3];
			
			fh.face_count++;
			
			fscanf(s, "%d:", &v);
			fscanf(s, " A:\t%d B:\t%d C:\t%d", &vv[0], &vv[1], &vv[2]);

			bfm_bin.WriteSig(SIG_FACE);
			bfm_bin.WriteInt16((PBFMINT16)vv, 3);
			printf("* face %d: %d, %d, %d\n", v, vv[0], vv[1], vv[2]);
		}
		else if (!strcmp(data, TVERTEX))
		{
			short v;
			float f[2];

			bfm_bin.WriteSig(SIG_TVERTEX);

			// we ignore the z value
			fscanf(s, "%d", &v);
			
			fscanf(s, "%f %f", &f[0], &f[1]);
			bfm_bin.WriteFloat32((BFMFLOAT32*)f, 2);
		}
		else if (!strcmp(data, TFACE))
		{
			short v;
			short l[3];
			
			bfm_bin.WriteSig(SIG_TFACE);
			
			fscanf(s, "%d:", &v);
			
			fscanf(s, "%d %d %d", &l[0], &l[1], &l[2]);
			bfm_bin.WriteInt16((BFMINT16*)l, 3);
		}
	
//////////////////////////////// MATERIALS ///////////////////////////////////////
		// read the current material number.
		else if (!strcmp(data, MAT_MATERIAL))
		{
			short v;
			
			fscanf(s, "%d", &v);
			
			fh.mat_count++;
			bfm_bin.WriteSig(SIG_MATERIAL);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
		}
		// read material color
		else if (!strcmp(data, MAT_AMBIENT))
		{
			float f[3];
			
			fscanf(s, "%f %f %f", &f[0], &f[1], &f[2]);
			
			bfm_bin.WriteSig(SIG_MATERIAL_AMBIENT);
			bfm_bin.WriteFloat32((PBFMFLOAT32)f, 3);		
		}
		else if (!strcmp(data, MAT_DIFFUSE))
		{
			float f[3];
			
			fscanf(s, "%f %f %f", &f[0], &f[1], &f[2]);
			
			bfm_bin.WriteSig(SIG_MATERIAL_DIFFUSE);
			bfm_bin.WriteFloat32((PBFMFLOAT32)f, 3);
		}
		else if (!strcmp(data, MAT_SPECULAR))
		{
			float f[3];
			
			fscanf(s, "%f %f %f", &f[0], &f[1], &f[2]);
			
			bfm_bin.WriteSig(SIG_MATERIAL_SPECULAR);
			bfm_bin.WriteFloat32((PBFMFLOAT32)f, 3);
		}

		// read texture
		else if (!strcmp(data, TEXTURE))
		{
			GetTextureName(s);
		}

		else if (!strcmp(data, MAT_NUMSUBMTL))
		{
			short v;
			fscanf(s, "%d", &v);

			bfm_bin.WriteSig(SIG_NUMSUBMTL);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
		}
		else if (!strcmp(data, MAT_SUBMATERIAL))
		{
			short v;
			fscanf(s, "%d", &v);

			bfm_bin.WriteSig(SIG_SUBMATERIAL);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
		}
		// read MTLID of a face
		else if (!strcmp(data, MESH_MTLID))
		{
			short v;
			
			fscanf(s, " %d", &v);
			bfm_bin.WriteSig(SIG_MTLID);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
		}

		// read material reference number.
		else if (!strcmp(data, MAT_REFERENCE))
		{
			short v;
			BFMINT32 save_pos;
			
			fscanf(s, "%d", &v);
			
			// write object_count and mat_count
			bfm_bin.WriteCurrent(&save_pos);
			bfm_bin.WriteSeek(reserve_mat_ref);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
			bfm_bin.WriteSeek((BFMINT32)save_pos);
		}

//////////////////////////// GROUP //////////////////////////////
		else if (!strcmp(data, GROUP))
		{
			char buf[32];
			
			ZeroMemory((void*)buf, 32);

			fscanf(s, " \"%s", buf);
			
			// to remove "
			buf[strlen(buf)-1] = NULL;
			
			bfm_bin.WriteSig(SIG_GROUP);
			bfm_bin.WriteInt8((PBFMINT8)buf, 32);

			printf("SG: [ %s ]\n", buf);
			group_array[group_count] = group_range_count;
			group_count++;
		}

		else if (!strcmp(data, "{"))
		{
			group_range_count++;
		}
		else if (!strcmp(data, "}"))
		{
			group_range_count--;

			if (group_array[group_count-1] == group_range_count &&
				group_count > 0)
			{
				group_count--;
				bfm_bin.WriteSig(SIG_ENDGROUP);
			}
		}
	}
}


void aseConv(char* fn)
{
	FILE *s;

	if (!(s = fopen(fn, "r")))
	{
		// file not found, bummer
		printf("# %s not found!\n", fn);
		return;
	}

	aseParse(s);

	fclose(s);
}

void main(int argc, char* argv[])
{
	printf("ASEC - ASE Converter\n");

	if (argc < 2)
		return;

	bfm_bin.Init("out.bin");
	bfm_bin.OpenToWrite();

	// save a room for file header information
	BFMINT32	savepos;
	bfm_bin.WriteCurrent(&savepos);
	bfm_bin.WriteSkip(sizeof(FILE_HEADER));

	aseConv(argv[1]);

	// write header
	bfm_bin.WriteSeek(savepos);
	bfm_bin.WriteInt8((PBFMINT8)&fh, sizeof(FILE_HEADER));
}

