

/*
**	Math Tools.  Jinhong Kim, cheriz@empal.com
**
**  THIS IS COMPLETELY FREE!  ANYONE WILL BE PERMITTED TO MODIFY THE CODE FOR HIS/HER OWN PURPOSE.
**  (THE PURPOSE INCLUDES THE COMMERCIAL)
**  BUT, YOU SHOULD KNOW THAT I'M FREE FROM ANY-RESPONSIBILITIES FOR- 
**  ANY SYSTEM RELATED ERRORS MAY OCCUR WHILE USING THE CODE.
**  FREE YOUR MIND.
*/



#ifndef __MATHMODULE__
#define __MATHMODULE__


#include <math.h>

#pragma warning (disable : 4201)	// nonstandard extension used : nameless struct/union
#pragma warning (disable : 4710)	// not inlined warning


class w3DPoint3
{
public:
	union
	{
		struct
		{
			float x, y, z;
		};
		float ar[3];
	};

	w3DPoint3()
	{
		x = 0;
		y = 0;
		z = 0;
	}

	w3DPoint3(float _x, float _y, float _z)
	{
		x = _x;
		y = _y;
		z = _z;
	}

	~w3DPoint3() {}

	void Set(float dx, float dy, float dz)
	{
		x = dx;
		y = dy;
		z = dz;
	}

	void Set(w3DPoint3 p)
	{
		x = p.x;
		y = p.y;
		z = p.z;
	}
};


class w3DPoint2
{
public:
	union
	{
		struct
		{
			float x, y;
		};
		float ar[2];
	};

	w3DPoint2()
	{
		x = 0;
		y = 0;
	}

	w3DPoint2(float _x, float _y)
	{
		x = _x;
		y = _y;
	}

	~w3DPoint2() {}

	void Set(float dx, float dy)
	{
		x = dx;
		y = dy;
	}

	void Set(w3DPoint2 p)
	{
		x = p.x;
		y = p.y;
	}
};


class w3DVector3
{
public:
	union
	{
		struct
		{
			float x, y, z;
		};
		float ar[3];
	};

	w3DVector3(float xx, float yy, float zz)
	{
		x = xx;
		y = yy;
		z = zz;
	}

	w3DVector3()
	{
		x = 0;
		y = 0;
		z = 0;
	}

	w3DVector3 Cross(w3DVector3 b)
	{
		w3DVector3 c(y*b.z - z*b.y, z*b.x - x*b.z, x*b.y - y*b.x);
		return c;
	}	

	~w3DVector3() {}

	void Set(float dx, float dy, float dz)
	{
		x = dx;
		y = dy;
		z = dz;
	}

	void Set(w3DVector3 v)
	{
		x = v.x;
		y = v.y;
		z = v.z;
	}

	void Flip()
	{
		x = -x;
		y = -y;
		z = -z;
	}

	void SetDiff(w3DPoint3 a, w3DPoint3 b)
	{
		x = a.x - b.x;
		y = a.y - b.y;
		z = a.z - b.z;
	}

	float GetLength()
	{
		double len = x*x + y*y + z*z;

		if (len < 0.0000001)
			return 1.0f;

		return (float)sqrt(len);
	}

	void Norm()
	{
		float sfactor = 1.0f / GetLength();

		x *= sfactor;
		y *= sfactor;
		z *= sfactor;
	}

	void Acc(w3DVector3 b)
	{
		x += b.x;
		y += b.y;
		z += b.z;
	}

	void Copy(w3DVector3 v)
	{
		x = v.x;
		y = v.y;
		z = v.z;
	}

	float Dot(w3DVector3 b)
	{
		return x*b.x + y*b.y + z*b.z;
	}
};


#pragma warning (default : 4201)
#pragma warning (default : 4710)


#endif

